// SPDX-License-Identifier: GPL-3.0-or-later

#pragma once

#include <stdint.h>

const uint16_t DSO602x_FW_VER = 0x0210;

// setup.py: __version__ = '2.10.6'
// PyHT6022/LibUsbScope.py: FIRMWARE_VERSION = 0x0210
// PyHT6022/Firmware/DSO6022BE/descriptor.inc: FIRMWARE_VERSION = 0x1002

